@@ -1,7 +1,7 @@
 #' Pareto smoothed importance sampling (PSIS)
 #'
 #' @export
-#' @param lw A matrix or vector of log weights. For computing LOO \code{lw =
+#' @param lw A matrix or vector of log weights. For computing LOO, \code{lw =
 #'   -log_lik} (see \code{\link{extract_log_lik}}) and is an \eqn{S} by \eqn{N}
 #'   matrix where \eqn{S} is the number of simulations and \eqn{N} is the number
 #'   of data points. (If \code{lw} is a vector it will be coerced to a
@@ -13,24 +13,21 @@
 #' @param wtrunc For truncating very large weights to \eqn{S}^\code{wtrunc}. Set
 #'   to zero for no truncation.
 #' @param cores The number of cores to use for parallelization. This can be set
-#'   for an entire R session by \code{options(loo.cores = NUMBER)}. The default is
-#'   \code{\link[parallel]{detectCores}}().
+#'   for an entire R session by \code{options(loo.cores = NUMBER)}. The default
+#'   is \code{\link[parallel]{detectCores}}().
 #' @param llfun,llargs See \code{\link{loo.function}}.
 #' @param ... Ignored when \code{psislw} is called directly. The \code{...} is
 #'   only used internally when \code{psislw} is called by the \code{\link{loo}}
 #'   function.
 #'
-#' @return A named with list with components \code{lw_smooth} (modified log
-#'   weights) and \code{pareto_k} (estimated generalized Pareto shape parameters
+#' @return A named list with components \code{lw_smooth} (modified log weights)
+#'   and \code{pareto_k} (estimated generalized Pareto shape parameter(s)
 #'   \eqn{k}).
 #'
 #' @details See the 'PSIS-LOO' section in \code{\link{loo-package}}.
 #'
-#' @note This function is primarily intended for internal use, but is exported
-#'   so that users can call it directly for other purposes. Users simply
-#'   wishing to compute LOO should use the \code{\link{loo}} function.
-#'
-#' @seealso \code{\link{loo-package}}, \code{\link{loo}}
+#' @template internal-function-note
+#' @template loo-paper-reference
 #'
 #' @importFrom matrixStats logSumExp
 #' @importFrom parallel mclapply makePSOCKcluster stopCluster parLapply
@@ -49,6 +46,9 @@ psislw <- function(lw, wcp = 0.2, wtrunc = 3/4,
     tail_len <- length(x_tail)
     if (tail_len < MIN_TAIL_LENGTH) {
       # too few tail samples to fit gPd
+      warning("Too few tail samples to fit generalized Pareto distribution.\n",
+              "Weights are truncated and normalized but not smoothed.",
+              call. = FALSE)
       x_new <- x
       k <- Inf
     } else {
@@ -74,18 +74,16 @@ psislw <- function(lw, wcp = 0.2, wtrunc = 3/4,
 
   .psis_loop <- function(i) {
     if (LL_FUN) {
-      ll_i <- llfun(i = i, data = llargs$data[i,,drop=FALSE], draws = llargs$draws)
+      ll_i <- llfun(i = i, data = llargs$data[i,, drop=FALSE],
+                    draws = llargs$draws)
       lw_i <- -1 * ll_i
     } else {
       lw_i <- lw[, i]
       ll_i <- -1 * lw_i
     }
     psis <- .psis(lw_i)
-    if (FROM_LOO) {
-      lse <- logSumExp(ll_i + psis$lw_new)
-      nlist(lse, k = psis$k)
-    }
-    else list(lw_new = psis$lw_new, k = psis$k)
+    if (!FROM_LOO) psis
+    else nlist(lse = logSumExp(ll_i + psis$lw_new), k = psis$k)
   }
 
   # minimal cutoff value. there must be at least 5 log-weights larger than this
@@ -97,23 +95,27 @@ psislw <- function(lw, wcp = 0.2, wtrunc = 3/4,
     dots$COMPUTE_LOOS else FALSE
 
   if (!missing(lw)) {
-    if (!is.matrix(lw))
-      lw <- as.matrix(lw)
+    if (!is.matrix(lw)) lw <- as.matrix(lw)
     N <- ncol(lw)
     LL_FUN <- FALSE
   } else {
     if (is.null(llfun) || is.null(llargs))
-      stop("Either lw or llfun and llargs must be specified.")
+      stop("Either 'lw' or 'llfun' and 'llargs' must be specified.")
     N <- llargs$N
     LL_FUN <- TRUE
   }
-
-  if (.Platform$OS.type != "windows") {
-    out <- mclapply(X = 1:N, FUN = .psis_loop, mc.cores = cores)
+  if (cores == 1) {
+    # don't call functions from parallel package if cores=1
+    out <- lapply(X = 1:N, FUN = .psis_loop)
   } else {
-    cl <- makePSOCKcluster(cores)
-    on.exit(stopCluster(cl))
-    out <- parLapply(cl, X = 1:N, fun = .psis_loop)
+    # parallelize
+    if (.Platform$OS.type != "windows") {
+      out <- mclapply(X = 1:N, FUN = .psis_loop, mc.cores = cores)
+    } else {
+      cl <- makePSOCKcluster(cores)
+      on.exit(stopCluster(cl))
+      out <- parLapply(cl, X = 1:N, fun = .psis_loop)
+    }
   }
   pareto_k <- vapply(out, "[[", 2L, FUN.VALUE = numeric(1))
   if (FROM_LOO) {
